// Licensed under the Apache License, version 2.0.
package com.smies.hornetq;

import java.util.Set;

import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectInstance;
import javax.management.ObjectName;

import org.jboss.system.ServiceMBeanSupport;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Provides a hook between the JBoss MC and the JMX Kernel enabling the possiblity to create
 * a dependency between HornetQ and packages, based on the JMX Kernel, that need the HornetQ server to be deployed.
 *
 * This is a workaround for the following issue: https://jira.jboss.org/jira/browse/HORNETQ-339
 *
 * When stopping the JBoss AS Server this MBean also makes sure that the hornetq-ra.rar deployment is stopped.
 *
 * Note: This service does not support re-deployment. When the stop process is executed then the JBoss server must be
 *       restarted.
 *
 * @author maurice_zeijen
 */
public class HornetQMBeanHook extends ServiceMBeanSupport implements HornetQMBeanHookMBean {

	private static final Logger logger = LoggerFactory.getLogger(HornetQMBeanHook.class);

	private MBeanServer mBeanServer;

	public HornetQMBeanHook(MBeanServer mBeanServer) {
		this.mBeanServer = mBeanServer;
	}

	@Override
	public void stopService() {
		try {
			if (!stopHornetQRar()) {
				stopJmsRaRar();
			}
		} catch (Exception e) {
			logger.error("Unable to stop HornetQ resource adapter.", e);
		}
	}

	private boolean stopHornetQRar() throws MalformedObjectNameException {
		return stopMBeanService(new ObjectName("jboss.jca:name='hornetq-ra.rar',service=RARDeployment"));
	}

	private boolean stopJmsRaRar() throws MalformedObjectNameException {
		return stopMBeanService(new ObjectName("jboss.jca:name='jms-ra.rar',service=RARDeployment"));
	}

	private boolean stopMBeanService(ObjectName objectName) {
		try {
	      Set<ObjectInstance> mbeanSet = mBeanServer.queryMBeans(objectName, null);

	      for (ObjectInstance mbean : mbeanSet) {
	        String stateString = (String)mBeanServer.getAttribute(mbean.getObjectName(), "StateString");

	        if ("Started".equalsIgnoreCase(stateString) || "Starting".equalsIgnoreCase(stateString)) {
	        	mBeanServer.invoke(mbean.getObjectName(), "stop", new Object[0], new String[0]);
	        }
	      }
	      return (mbeanSet.size() > 0);
	    }
	    catch (Exception e) {
	      throw new RuntimeException("Failed to invoke the 'stop' method on the listener object '" + objectName + "'", e);
	    }
	}

}
